# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import xml.parsers.expat

from util.UserException import UserException
from util.File import File

from Task import Task
from Library import *
from Song import *
from Artist import *
from Tune import *
from Disc import *

class LibraryParserException(UserException):
    """An exception raised when an error occurs during the parsing."""

    __file_path = ''

    __line = 0

    __column = 0

    __code = 0

    __message = ""

    def __init__(self, p_file_path, p_line, p_column, p_code, p_message):
        UserException.__init__(self)
        self.__file_path = p_file_path
        self.__code = p_code
        self.__message = p_message
        self.__line = p_line
        self.__column = p_column

    def get_file_path(self):
        """Return the file path."""
        return self.__file_path

    def get_line(self):
        """Return the line."""
        return self.__line

    def get_column(self):
        """Return the column."""
        return self.__column

    def get_code(self):
        """Return the code."""
        return self.__code

    def get_message(self):
        """Return the message."""
        return self.__message

    def __str__(self):
        """Return a representation as a string."""
        _ret = ''
        _ret += self.__file_path + ' - ' + str(self.__line) + ' - ' + str(self.__column)
        _ret += ' - ' + str(self.__code) + " : " + self.__message
        return _ret


class LibraryLoading(Task):
    """A parser which adds songs into a library from a XML file."""

    __library = None

    __file_path = ''


    # attributes for the parser

    __parser = None

    __current_file_name = ''

    __current_song = None

    __current_artist = None

    __tags = []

    __current_data = ''

    def __init__(self, p_library, p_file_path):
        Task.__init__(self)
        self.__library = p_library
        self.__file_path = p_file_path
        self.__parser = xml.parsers.expat.ParserCreate('ISO-8859-1')
#        self.__parser.returns_unicode = 0
        self.__parser.StartElementHandler = self.__start_element
        self.__parser.EndElementHandler = self.__end_element
        self.__parser.CharacterDataHandler = self.__char_data
        self.__current_file_name = ''
        self.__current_song = None
        self.__current_artist = None
        self.__tags = []
        self.__current_data = ''


    def get_file_path(self):
        return self.__file_path


    def start(self):
        """Load the library from the file.

        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable."""
        # clear the library
        self.__library.remove_all_songs()
        # open the library file
        _file = File(self.get_file_path())
        _f = _file.open_to_read()
        # parse the file
        self.__parser.ParseFile(_f)
        # get the error code
        _num = self.__parser.ErrorCode
        if _num != 0:
            _msg = xml.parsers.expat.ErrorString(_num)
            _line = self.__parser.ErrorLineNumber
            _col = self.__parser.ErrorColumnNumber
            raise LibraryParserException(self.__file_path, _line, _col, _num, _msg)
        # close file
        _f.close()

    def __is_parent_tag(self, p_tag):
        _tag = ""
        if len(self.__tags) >= 2:
            _tag = self.__tags[-2]
        return (_tag == p_tag)

    def __is_last_tag(self, p_tag):
        _tag = ""
        if len(self.__tags) >= 1:
            _tag = self.__tags[-1]
        return (_tag == p_tag)

    def __start_element(self, p_name, p_attrs):
        """Handle a start element."""
#        print 'Start element:', p_name, p_attrs
        if p_name == LIBRARY_TAG:
            _root_path = p_attrs[ROOT_PATH_ATT].encode("ISO-8859-1")
            # change current directory
            _dir = Directory(_root_path)
            _dir.change()
            # set root path
            self.__library.set_root_path(_root_path)
        elif p_name == FILE_TAG:
            self.__current_file_name = p_attrs[FILE_NAME_ATT].encode("ISO-8859-1")
#            self.__current_file_name = p_attrs[FILE_NAME_ATT]
        elif p_name == SONG_TAG:
            self.__current_song = get_empty_song()
        self.__tags.append(p_name)
        self.__current_data = ""


    def __end_element(self, p_name):
        """Handle an end element."""
#        print 'End element:', p_name
        if p_name == FILE_TAG:
#            print 'File : ', self.__current_file_name
#            print 'Song : ', self.__current_song
            _file = File(self.__current_file_name)
#            print self.__current_file_name, 'found'
            if _file.exists():
                self.__library.add_song(self.__current_file_name, self.__current_song)
#                print 'Added'
#                print self.__current_file_name, 'added with song :'
#                print self.__current_song
        elif self.__is_parent_tag(ARTIST_TAG) and self.__is_last_tag(ARTIST_NAME_TAG):
            if self.__current_artist is None:
                self.__current_artist = Artist(self.__current_data)
            else:
                self.__current_artist.add_name(self.__current_data)
        elif self.__is_parent_tag(ARTIST_TAG) and self.__is_last_tag(FEAT_ARTIST_NAME_TAG):
            if self.__current_artist is not None:
                self.__current_artist.add_feat_name(self.__current_data)
        elif self.__is_parent_tag(TUNE_TAG) and self.__is_last_tag(TITLE_TAG):
            self.__current_song.get_tune().set_title(self.__current_data)
        elif self.__is_parent_tag(TUNE_TAG) and self.__is_last_tag(YEAR_TAG):
            self.__current_song.get_tune().set_year(self.__current_data)
        elif self.__is_parent_tag(DISC_TAG) and self.__is_last_tag(DISC_TITLE_TAG):
            self.__current_song.get_disc().set_title(self.__current_data)
        elif self.__is_parent_tag(DISC_TAG) and self.__is_last_tag(ARTIST_TAG):
            self.__current_song.get_disc().set_artist(self.__current_artist)
            self.__current_artist = None
        elif self.__is_last_tag(ARTIST_TAG):
            self.__current_song.set_artist(self.__current_artist)
            self.__current_artist = None
        elif self.__is_last_tag(TRACK_TAG):
            self.__current_song.set_track(self.__current_data)
        elif self.__is_last_tag(GENRE_TAG):
            self.__current_song.set_genre(self.__current_data)
        elif self.__is_last_tag(COMMENT_TAG):
            self.__current_song.set_comment(self.__current_data)

#        self.__last_tag = self.__parent_tag
#        self.__parent_tag = ""
        self.__tags.pop()
        self.__current_data = ""


    def __char_data(self, p_data):
        """Handle a character data."""
#        print 'Character data:', repr(p_data)
        self.__current_data += p_data.encode("ISO-8859-1")
#        self.__current_data += p_data


if __name__ == '__main__':
    _lib = Library()
    _loading = LibraryLoading(_lib, 'library.xml')
    _loading.start()
    print _lib
