# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import Directory
from util.File import FileNotReadableException

from Task import Task
from Process import BadStatusException
from TagsReading import TAGS_READING_TOOL
from TagsReading import TAGS_READING_TOOL_EXISTS
from TagsReading import TagsReading
from MusicFile import FileNotOggVorbisException
from SongFile import SongFile


def get_library_refreshing_tools():
    return [TAGS_READING_TOOL]

def can_refresh_library():
    return TAGS_READING_TOOL_EXISTS

class LibraryRefreshing(Task):
    """A task which refresh entirely the library from a root directory."""

    __manager = None

    __library = None

    __root_path = ''

#    __play_lists_set = None

    __tags_reading = None

    __stop = 0

    def __init__(self, p_manager, p_library, p_root_path):
        Task.__init__(self)
        self.__manager = p_manager
        self.__library = p_library
        self.__root_path = p_root_path
#        self.__play_lists_set = p_play_lists_set
        self.__tags_reading = None
        self.__stop = 0

    def get_root_path(self):
        return self.__root_path

    def get_current_file_path(self):
        _ret = None
        if self.__tags_reading is not None:
            _ret = self.__tags_reading.get_file_path()
        return _ret

    def __refresh_play_lists(self):
        """Update all songs of the play lists."""
        _play_lists = self.__manager.get_play_lists_set() + self.__manager.get_play_thread().get_queue()
        for _pl in _play_lists:
            for _file in _pl:
                _path = _file.get_path()
                if self.__library.has_song(_path):
                    _file.set_song(self.__library.get_song(_path))
                else:
                    _file.set_song(None)

    def __explore(self, p_dir):
        """Browse the given directory for reading all Ogg Vorbis files
        and adding to the library.

        Raise ToolNotFoundException if the tool for reading tags is not available."""
        _entries = p_dir.list()
#        print "p_dir :", p_dir
#        print "_entries :", _entries
        for _inode in _entries:
            # if the task must stop, stop the recusivity
            if self.__stop:
                break
#            print "_inode :", _inode
            if _inode.is_directory():
#                print "_inode is a dir"
                # the entry is a directory
                _dir = Directory(_inode.get_path())
                self.__explore(_dir)
            elif _inode.is_file():
#                print "_inode is a file"
                # the entry is a file
                # read tags
                self.__tags_reading = TagsReading(_inode.get_path())
                try:
                    _song = self.__tags_reading.start()
                except BadStatusException:
                    pass
                except FileNotReadableException:
                    pass
                except FileNotOggVorbisException:
                    pass
                else:
                    # adds the song to the library
                    self.__library.add_song(_inode.get_path(), _song)
                

    def start(self):
        """Refresh the library from the root path.

        Raise ToolNotFoundException if the tool for reading tags is not available.
        Raise DirectoryNotFoundException if the directory does not exist.
        Raise DirectoryNotAccessibleException if the directory cannot be accessed."""
        # clear the library
        self.__library.remove_all_songs()
        # change directory
        _root_path = self.get_root_path()
        self.__library.set_root_path(_root_path)
        _dir = Directory(_root_path)
        _dir.change()
        # explore recursively all directories
        _dir = Directory("")
        self.__explore(_dir)
        # refresh play lists
        self.__refresh_play_lists()


    def stop(self):
        Task.stop(self)
        self.__stop = 1
        if self.__tags_reading is not None:
            self.__tags_reading.stop()



if __name__ == '__main__':
    import sys
    import string
    from Manager import Manager
    from Library import Library
    if not can_refresh_library():
        print 'Required tools : %s' % string.join(get_library_refreshing_tools())
    elif len(sys.argv) != 2:
        print 'Usage : %s %s directory' % (sys.executable, sys.argv[0])
    else:
        _manager = Manager()
        _lib = Library()
        _dir_path = sys.argv[1]
        _task = LibraryRefreshing(_manager, _lib, _dir_path)
        print 'Refreshing the library from the directory :'
        print '\t%s' % _task.get_root_path()
        print 'Please wait...'
        _task.start()
        print _lib

