# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import threading
import tempfile

from util.File import File
from util.Directory import Directory

from Library import Library
from LogFile import LogFile
from TasksThread import TasksThread
from PlayThread import PlayThread
from PlayListsSet import PlayListsSet
from PathConfig import PathConfig
from Process import Process
from MP3Importing import MP3Importing
from TrackImporting import TrackImporting
from SongAdding import SongAdding
from SongEditing import SongEditing
from SongRemoving import SongRemoving
from LibraryRefreshing import LibraryRefreshing
from LibraryLoading import LibraryLoading
from LibrarySaving import LibrarySaving
from PlayListsSaving import PlayListsSaving
from PlayListsLoading import PlayListsLoading
from PathConfigSaving import PathConfigSaving
from PathConfigLoading import PathConfigLoading


class Manager:
    """The single Ogg Vorbis files manager in the application.

    This class is synchronized so that several threads
    can use a same instance."""

    __manager = None

    def __init__(self):
        if not Manager.__manager:
            Manager.__manager = Manager.__Manager()

    def __getattr__(self, p_name):
        return getattr(self.__manager, p_name)

    class __Manager:
        """An Ogg Vorbis files manager."""

    #    __root_dir = None

        __library = None

        __tasks_thread = None

        __play_thread = None

        __play_lists_set = None

        __path_config = None

        __tmp_path = ''

        __lock = None # lock used only for synchronized path config and tmp directory

        def __init__(self): #, p_root_path):
            """Construct an Ogg Vorbis files manager with the given directory path
            as a root Ogg Vorbis directory."""
    #        self.__root_dir = Directory(p_root_path)
            self.__library = Library()
            self.__tasks_thread = TasksThread()
            self.__play_thread = PlayThread()
            self.__play_lists_set = PlayListsSet()
            self.__path_config = PathConfig()
            self.__tmp_path = tempfile.gettempdir()
            self.__lock = threading.Lock()

        def __get_conf_dir(self):
            """Return the configuration directory.

            Raise DirectoryNotFoundException if the home directory does not exist.
            Raise DirectoryNotWritableException if the home directory is not writable."""
            _dir = Directory(os.environ["HOME"])
            _dir = _dir.get_directory(".maestro")
            # create the directory if necessary
            if not _dir.exists():
                _dir.create()
            return _dir

        #################################################
        # Accessors
        #################################################

    #    def get_song(self, p_file_path):
    #        """Return the song corresponding
    #        of the given file path in the library.
    #        Return None if the file path is not in the library."""
    #        _ret = None
    #        try:
    #            _ret = self.__library.get_song(p_file_path)
    #        except KeyError:
    #            _ret = None
    #        return _ret

        def get_library_selection(self):
            """Return the library as a selection."""
            _sel = self.__library.get_selection()
            return _sel

        def get_tasks_thread(self):
            """Return the tasks thread."""
            return self.__tasks_thread

        def get_play_thread(self):
            """Return the play thread."""
            return self.__play_thread

        def get_play_lists_set(self):
            """Return the play lists set."""
            return self.__play_lists_set

        def get_path_config(self):
            """Return the current path configuration."""
            self.__lock.acquire()
            _ret = self.__path_config
            self.__lock.release()
            return _ret

        def get_tmp_path(self):
            """Return the temporary directory path."""
            self.__lock.acquire()
            _ret = self.__tmp_path
            self.__lock.release()
            return _ret

        def get_root_path(self):
            """Return the temporary directory path."""
            return self.__library.get_root_path()

        #################################################
        # Modificators
        #################################################

        def set_path_config(self, p_config):
            """Set the current path configuration."""
            self.__lock.acquire()
            self.__path_config = p_config
            self.__lock.release()

        def set_tmp_path(self, p_tmp_path):
            """Set the temporary directory path."""
            self.__lock.acquire()
            self.__tmp_path = p_tmp_path
            self.__lock.release()

        #################################################
        # Log management
        #################################################

        def enable_logs(self):
            """Activate logs."""
            _dir = self.__get_conf_dir()
            _path = _dir.get_file_path("io.log")
            Process.IO_LOG_FILE = LogFile(_path)
            _path = _dir.get_file_path("play.log")
            Process.PLAY_LOG_FILE = LogFile(_path)

        #################################################
        # Library management
        #################################################

        def __get_library_file(self):
            """Return the library file of user configuration directory."""
            _dir = self.__get_conf_dir()
            _file = File(_dir.get_file_path("library.xml"))
            return _file

        def get_library_saving(self):
            """Return a task which saves the library."""
            _path = self.__get_library_file().get_path()
            _task =  LibrarySaving(self.__library, _path)
            return _task

        def get_library_loading(self):
            """Return a task which loads the library."""
            _path = self.__get_library_file().get_path()
            _task =  LibraryLoading(self.__library, _path)
            return _task


        #################################################
        # Path configuration management
        #################################################

        def __get_path_config_file(self):
            """Return the library file of user configuration directory."""
            _dir = self.__get_conf_dir()
            _file = File(_dir.get_file_path("path.conf"))
            return _file

        def get_path_config_saving(self):
            """Return a task which saves the library."""
            _path = self.__get_path_config_file().get_path()
            _task =  PathConfigSaving(self.__path_config, _path)
            return _task

        def get_path_config_loading(self):
            """Return a task which loads the library."""
            _path = self.__get_path_config_file().get_path()
            _task =  PathConfigLoading(self, _path)
            return _task

        #################################################
        # Files and library management
        #################################################

        def get_mp3_importing(self, p_file_path, p_song):
            """Return a task which imports a MP3 file."""
            # create the task
            _task = MP3Importing(self, self.__library, p_file_path, p_song)
            return _task

        def get_track_importing(self, p_track, p_song):
            """Return a task which imports a CDDA track."""
            # create the task
            _task = TrackImporting(self, self.__library, p_track, p_song)
            return _task

        def get_song_adding(self, p_file_path, p_song):
            """Return a task which imports an Ogg Vorbis file."""
            # create the task
            _task = SongAdding(self, self.__library, p_file_path, p_song)
            return _task

        def get_song_editing(self, p_file_path, p_song):
            """Return a task which edits an Ogg Vorbis file."""
            # create the task
            _task = SongEditing(self, self.__library, p_file_path, p_song)
            return _task

        def get_song_removing(self, p_file_path):
            """Return a task which removes an Ogg Vorbis file."""
            # create the task
            _task = SongRemoving(self, self.__library, p_file_path)
            return _task

        def get_library_refreshing(self, p_root_path):
            """Return a task which refreshs the library
            from the given root directory path."""
            # create the task
            _task = LibraryRefreshing(self, self.__library, p_root_path)
            return _task


        #################################################
        # Play lists management
        #################################################

        def __get_play_lists_dir(self):
            """Return the directory where playlists are saved."""
            _dir = self.__get_conf_dir()
    #        _dir = Directory(_dir_name)
            _dir = _dir.get_directory("playlists")
            if not _dir.exists():
                _dir.create()
            return _dir        

        def get_play_lists_loading(self):
            """Return a task which loads all playlists."""
            _dir = self.__get_play_lists_dir()
            _task = PlayListsLoading(self.__play_lists_set, self.__library, _dir.get_path())
            return _task


        def get_play_lists_saving(self):
            """Return a task which saves all playlists."""
            _dir = self.__get_play_lists_dir()
            _task = PlayListsSaving(self.__play_lists_set, _dir.get_path())
            return _task
