# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import Directory
from SongFile import SongFile
from Song import Song
from Tune import Tune
from Artist import Artist
from Disc import Disc

FORMAT = 'FORMAT'
DISC_FORMAT = 'DISC_FORMAT'
ARTISTS_SEP = 'ARTISTS_SEP'
SPACE_SUB = 'SPACE_SUB'
LOWERCASE = 'LOWERCASE'

def get_complex_example(p_path_config):
    """Return a file path from a complex example."""
    _tune = Tune("Song title", "1981")
    _artist = Artist("First artist")
    _artist.add_name("Second artist")
    _artist.add_feat_name("Third artist")
    _disc = Disc("Disc title", _artist.get_main_artist())
    _song = Song(_artist, _tune, _disc, "7", "Misc", "Example")
    return p_path_config.get_correct_file(_song)

def get_simple_example(p_path_config):
    """Return a file path from a simple example,
    ie with a no artist disc and a single artist song."""
    _tune = Tune("Song title", "1981")
    _artist = Artist("Song artist")
    _disc = Disc("Disc title")
    _song = Song(_artist, _tune, _disc, "7", "Misc", "Example")
    return p_path_config.get_correct_file(_song)


class PathConfig:
    """An Ogg Vorbis path preference.

    Warning ! For a reason of performance, this class is not synchronized.
    Don't apply its modificators to a shared instance.

    Ogg Vorbis file relative paths is given by applying formats and rules.
    The main format matches special substring for replacing them by a song
    attribute as the following table :

    Substring | Attribute
    -----------------------
       %t     | title
       %y     | year
       %d     | disc
       %a     | artist(s)
       %r     | track
       %g     | genre

    When the disc has no artist, %d is just replaced by the disc title.
    When the dis $c has an artist, the disc format is applied with these
    substring replacements :

    Substring | Attribute
    -----------------------
       %t     | title
       %a     | artist

    In all formats, %% will be replaced by %.

    BUG : if an attribute contains special substring, the special substring will be replaced.
    """

    __root_dir = None

    __tmp_dir = None

    __format = '%d/%a - %t.ogg'

    __disc_format = '%a - %t'

    __artists_sep = ', '

    __space_substitute = '_'

    __lowercase = 0

    def __init__(self, p_format = '%d/%a - %t.ogg', p_disc_format = '%a - %t',
                 p_artists_sep = ", ", p_space_substitute = '_', p_lowercase = 0):
        self.__format = p_format
        self.__disc_format = p_disc_format
        self.__artists_sep = p_artists_sep
        self.__space_substitute = p_space_substitute
        self.__lowercase = p_lowercase

    def get_format(self):
        """Return the main fomat."""
        return self.__format

    def set_format(self, p_format):
        """Set the main format of a file path."""
        self.__format = p_format

    def get_disc_format(self):
        """Return the format of a disc in a file path."""
        return self.__disc_format

    def set_disc_format(self, p_disc_format):
        """Set the format of a disc in a file path."""
        self.__disc_format = p_disc_format

    def get_artists_sep(self):
        """Return the artists names separator."""
        return self.__artists_sep

    def set_artists_sep(self, p_sep):
        """Set the artists names separator."""
        self.__artists_sep = p_sep

    def get_space_substitute(self):
        """Return the string which will replace a space."""
        return self.__space_substitute

    def set_space_substitute(self, p_substitute):
        """Set the string which will replace a space."""
        self.__space_substitute = p_substitute

    def is_lowercasing(self):
        """Determine if the lowercasing is enable."""
        return self.__lowercase

    def set_lowercasing(self, p_bool):
        """Enable or disable the lowercasing."""
        self.__lowercase = p_bool

    def __get_artist_string(self, p_artist):
        """Return the formatted string of the given artist."""
        # get the main artist name
        _ret = p_artist.get_name(0)
        # add the other names
        _len = p_artist.get_names_number()
        for _cpt in range(1, _len):
            _ret += self.__artists_sep + p_artist.get_name(_cpt)
        # add the optional featuring names
        _len = p_artist.get_feat_names_number()
        if _len > 0:
            _ret += ' feat. '
            _ret += p_artist.get_feat_name(0)
            for _cpt in range(1, _len):
                _ret += self.__artists_sep + p_artist.get_feat_name(_cpt)
        # return the formatted string
        return _ret                

    def __get_disc_string(self, p_disc):
        """Return the formatted string of the given disc."""
        _ret = ''
        _title = p_disc.get_title()
        if p_disc.has_artist():
            _ret = self.__disc_format
            _ret = _ret.replace('%t', _title)
            _artist = self.__get_artist_string(p_disc.get_artist())
            _ret = _ret.replace('%a', _artist)
        else:
            _ret = _title
        return _ret

    def get_correct_file_path(self, p_song):
        """Return a correct file path for the given song."""
        _path = self.__format
        _path = _path.replace('%t', p_song.get_tune().get_title())
        _path = _path.replace('%y', p_song.get_tune().get_year())
        _path = _path.replace('%d', self.__get_disc_string(p_song.get_disc()))
        _artist = self.__get_artist_string(p_song.get_artist())
        _path = _path.replace('%a', _artist)
        _path = _path.replace('%r', p_song.get_track())
        _path = _path.replace('%g', p_song.get_genre())
        # substitute spaces
        _path = _path.replace(' ', self.__space_substitute)
        # lowercase the path
        if self.__lowercase:
            _path = _path.lower()
        # return the path
        return _path
        
    def get_correct_file(self, p_song):
        """Return a correct song file for the given song."""
        _path = self.get_correct_file_path(p_song)
        # build a music file and return it
        _file = SongFile(_path, p_song)
        return _file

    def has_correct_path(self, p_m_file):
        """Determine if a music file has a correct path."""
        _file = None
        _song = p_m_file.get_song()
        if _song is not None:
            _file = self.get_correct_file(_song)
        return (_file == p_m_file)

    def __get_line(self, p_tag, p_value):
        return p_tag + '=' + p_value + '\n'

    def __str__(self):
        _ret = ''
        _ret += self.__get_line(FORMAT, self.get_format())
        _ret += self.__get_line(DISC_FORMAT, self.get_disc_format())
        _ret += self.__get_line(ARTISTS_SEP, self.get_artists_sep())
        _ret += self.__get_line(SPACE_SUB, self.get_space_substitute())
        _ret += self.__get_line(LOWERCASE, str(self.is_lowercasing()))
        return _ret
            

if __name__ == '__main__':
    _config = PathConfig()
    print "Default configuration :"
    print _config
    print "   Simple example :", _config.get_simple_example()
    print "   Complex example :", _config.get_complex_example()
    _config.set_format('%d/%r-%a-%t')
    _config.set_disc_format('%a/%t')
    _config.set_artists_sep('-')
    _config.set_space_substitute('-')
    _config.set_lowercasing(1)
    print "Another configuration :"
    print _config
    print "   Simple example :", _config.get_simple_example()
    print "   Complex example :", _config.get_complex_example()
    _config.set_format('%g/%y/%a-%t')
    _config.set_disc_format('%t-%a')
    _config.set_artists_sep(',')
    _config.set_space_substitute(' ')
    _config.set_lowercasing(0)
    print "A third configuration :"
    print _config
    print "   Simple example :", _config.get_simple_example()
    print "   Complex example :", _config.get_complex_example()
    
