# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import popen2
import os

from util.Directory import Directory
from util.UserException import UserException


def is_in_path(p_tool):
    """Determine if the given tool is in the PATH variable."""
    _ret = 0
    # get the PATH variable
    _path_val = os.environ['PATH']
    if _path_val is None:
        _path_val = ''
    # split the PATH variable
    _paths = _path_val.split(os.pathsep)
    for _path in _paths:
        _dir = Directory(_path)
        _inode = _dir.get_inode(p_tool)
        if _inode.exists() and _inode.is_executable():
            _ret = 1
            break
    return _ret


class ProcessException(UserException):
    """A base class for all exceptions related to a process."""

    pass


class ProcessKilledException(ProcessException):
    """An exception raised when a process was killed."""

    pass


class BadStatusException(ProcessException):
    """An exception raised when a process stops abnormally ie, returns
    a bad status (different of 0)."""

    pass


class ToolNotFoundException(ProcessException):
    """An exception raised when a tool cannot be found."""

    pass


#class ProcessNotEndedException(ProcessException):
#    """An exception raised when a process is alive."""
#
#    pass


PROCESS_NOT_STARTED = 0
PROCESS_ALIVE = 1
PROCESS_KILLED = 2
PROCESS_FINISHED = 3

class Process:
    "A process."

    IO_LOG_FILE = None

    PLAY_LOG_FILE = None

    __popen = None

    __state = PROCESS_NOT_STARTED

    __log_file = None

    def __init__(self, p_cmd, p_log_file = None):
        "Launch the given command in a process."
        self.__popen = popen2.Popen3(p_cmd, 1)
        self.__state = PROCESS_ALIVE
        self.__log_file = p_log_file
        self.__write_log("### " + p_cmd + " ###\n")

    def __write_log(self, p_string):
        """Write the given string in the log file."""
        if self.__log_file is not None:
            self.__log_file.write(p_string)
        
    def readline(self):
        """Return a single line of the process output."""
        _ret = self.__popen.fromchild.readline()
        return _ret

    def readlines(self):
        """Return the entire process output."""
        _ret = ""
        _list = self.__popen.fromchild.readlines()
        for _line in _list:
            _ret += _line
        return _ret

    def write_and_close(self, p_string):
        """Pass the given string to the process."""
        self.__popen.tochild.write(p_string)
        self.__popen.tochild.close()

    def __end_process(self, p_string):
        """Read all lines of the process output.
        Write the given string and the end code in the log file."""
        _str = ""
        _list = self.__popen.fromchild.readlines()
        for _line in _list:
            _str += _line
        _str += p_string
        self.__write_log(_str)

    def wait(self):
        """Wait for the process end.

        Raise ProcessException if the process return an code different of 0."""
        _ret = self.__popen.wait()
        self.__end_process("Return code : " + str(_ret) + "\n")
        if self.is_killed():
            raise ProcessKilledException
        self.__state = PROCESS_FINISHED
        if _ret != 0:
            raise BadStatusException, _ret
#        return _ret

    def is_alive(self):
        return (self.__state == PROCESS_ALIVE)

    def is_killed(self):
        return (self.__state == PROCESS_KILLED)

    def kill(self):
        """Send a signal to end the process."""
        if self.is_alive():
            self.__state = PROCESS_KILLED
            os.kill(self.__popen.pid, 15)
            self.__end_process("Killed\n")
