# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from Artist import Artist
from Disc import Disc
from Tune import Tune
from util.String import String
from util.NumberString import NumberString

SONG_TAG = 'song'

TRACK_TAG = 'track'

GENRE_TAG = 'genre'

COMMENT_TAG = 'comment'


def get_empty_song():
    """Return an empty song."""
    _artist = Artist('')
    _disc = Disc('')
    _tune = Tune('')
    _song = Song(_artist, _tune, _disc)
    return _song


def get_complete_song(p_artist_name, p_title, p_album_title):
    """Return a song with the given artist name, tune title and album title."""
    _artist = Artist(p_artist_name)
    _tune = Tune(p_title)
    _disc = Disc(p_album_title, _artist)
    _song = Song(_artist, _tune, _disc)
    return _song


class Song:
    """A song.

    Warning ! For a reason of performance, this class is not synchronized.
    Don't apply its modificators to a shared instance.
    """

    __artist = None

    __tune = None

    __disc = None

    __track = NumberString('')

    __genre = String('')

    __comment = String('')

    def __init__(self, p_artist, p_tune, p_disc, p_track = '', p_genre = '', p_comment = ''):
        """Construct a song."""
        self.__artist = p_artist
        self.__tune = p_tune
        self.__disc = p_disc
        self.__track = NumberString(p_track)
        self.__genre = String(p_genre)
        self.__comment = String(p_comment)

    def get_artist(self):
        """Return the artist."""
        return self.__artist

    def set_artist(self, p_artist):
        """Set the artist."""
        self.__artist = p_artist

    def get_tune(self):
        """Return the tune."""
        return self.__tune

    def set_tune(self, p_tune):
        """Set the tune."""
        self.__tune = p_tune

    def get_disc(self):
        """Return the disc."""
        return self.__disc

    def set_disc(self, p_disc):
        """Set the disc."""
        self.__disc = p_disc

    def get_track(self):
        """Return the track."""
        return self.__track.data

    def set_track(self, p_track):
        """Set the track."""
        self.__track = NumberString(p_track)

    def get_genre(self):
        """Return the genre."""
        return self.__genre.data

    def set_genre(self, p_genre):
        """Set the genre."""
        self.__genre = String(p_genre)

    def get_comment(self):
        """Return the comment."""
        return self.__comment.data

    def set_comment(self, p_comment):
        """Set the comment."""
        self.__comment = String(p_comment)

    def clone(self):
        """Return a clone."""
        _new_artist = self.__artist.clone()
        _new_tune = self.__tune.clone()
        _new_disc = self.__disc.clone()
        _new_track = self.get_track()
        _new_genre = self.get_genre()
        _new_comment = self.get_comment()
        _song = Song(_new_artist, _new_tune, _new_disc, _new_track, _new_genre, _new_comment)
        return _song


    def matchs(self, p_song):
        """Determine if the song matchs with another song."""
        _ret = 1
        if p_song is not None:
            _ret = self.__comment.matchs(p_song.get_comment())
#            print "Comment :", _ret, ";", 
            if _ret:
                _ret = self.__genre.matchs(p_song.get_genre())
#                print "Genre :", _ret, ";", 
            if _ret:
                _ret = self.__track.matchs(p_song.get_track())
#                print "Track :", _ret, ";", 
            if _ret:
                _ret = self.__disc.matchs(p_song.get_disc())
#                print "Disc :", _ret, ";", 
            if _ret:
                _ret = self.__tune.matchs(p_song.get_tune())
#                print "Tune :", _ret, ";", 
            if _ret:
                _ret = self.__artist.matchs(p_song.get_artist())
#                print "Artist :", _ret, ";", 
#            print
        return _ret

    def __cmp__(self, p_song):
        """Compare with another song."""
        _ret = 1
        if p_song is not None:
            _ret = cmp(self.__tune, p_song.__tune)
            if _ret == 0:
                _ret = cmp(self.__artist, p_song.__artist)
            if _ret == 0:
                _ret = cmp(self.__disc, p_song.__disc)
            if _ret == 0:
                _ret = cmp(self.__track, p_song.__track)
            if _ret == 0:
                _ret = cmp(self.__genre, p_song.__genre)
            if _ret == 0:
                _ret = cmp(self.__comment, p_song.__comment)
        return _ret

    def __str__(self):
        """Return a representation as a string."""
        _ret = str(self.__tune) + " | "
        _ret += str(self.get_artist()) + " | "
        _ret += str(self.get_disc()) + " | "
        _ret += self.get_track() + " | "
        _ret += self.get_genre() + " | "
        _ret += self.get_comment()
        return _ret

    def write_xml(self, p_output):
        """Write a represetation as a XML string."""
        _output = p_output
        _output.write_begin_tag(SONG_TAG)
        _output = self.__artist.write_xml(_output)
        _output = self.__tune.write_xml(_output)
        _output = self.__disc.write_xml(_output)
        _output.write_tags_with_data(TRACK_TAG, self.get_track())
        _output.write_tags_with_data(GENRE_TAG, self.get_genre())
        _output.write_tags_with_data(COMMENT_TAG, self.get_comment())
        _output.write_end_tag(SONG_TAG)
        return _output



if __name__ == '__main__':
    import sys
    from util.XMLOutput import XMLOutput
    _artist = Artist('Jules')
    _disc = Disc('Greatest hits', _artist)
    _tune = Tune('Happy birthday', 1981)
    _song = Song(_artist, _tune, _disc, '7', 'classic', 'my first song')
    print "write_xml()"
    _output = XMLOutput(sys.stdout)    
    _song.write_xml(_output)
        
