# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import string

from Artist import NAMES_SEP
from Artist import FEAT_SEP
from Song import get_empty_song
from Disc import Disc
from Artist import Artist
from Tune import Tune
from Song import Song

ARTIST = ['ARTIST']
TITLE = ['TITLE']
ALBUM = ['ALBUM', 'DISC']
TRACKNUMBER = ['TRACKNUMBER', 'TRACK']
YEAR = ['YEAR', 'DATE']
GENRE = ['GENRE']
DESCRIPTION = ['DESCRIPTION', 'COMMENT']

EXTRA_SEP = ', '


class TagsSet:
    """A set of Ogg Vorbis tags."""

#    FEAT = " feat. "
#    SEP = ", "

    __tags = {}

    def __init__(self, p_song = None):
        """Construct a tags set."""
        self.__tags = {}
        if p_song is not None:
            # add tags for artist
            _artist = p_song.get_artist()
            _len = _artist.get_names_number()
            for _cpt in range(_len):
                _name = _artist.get_name(_cpt)
                self.add_tag(ARTIST[0], _name)
            # add the other tags
            self.add_tag(TITLE[0], p_song.get_tune().get_entire_title())
            self.add_tag(ALBUM[0], p_song.get_disc().get_title())
            self.add_tag(TRACKNUMBER[0], p_song.get_track())
            self.add_tag(YEAR[0], p_song.get_tune().get_year())
            self.add_tag(GENRE[0], p_song.get_genre())
            self.add_tag(DESCRIPTION[0], p_song.get_comment())
        
    def add_tag(self, p_key, p_data):
        """Add a tag with the given value and key."""
        _key = p_key.upper()
        if self.__tags.has_key(_key):
            self.__tags[_key].append(p_data)
        else:
            self.__tags[_key] = [p_data]
        
    def get_tag(self, p_key, p_sep = EXTRA_SEP):
        """Return the tag values of the given key
        separated by the given separator."""
        _ret = ''
        if self.__tags.has_key(p_key):
            _ret = string.join(self.__tags[p_key], p_sep)
        return _ret


    def __get_tags_in_one(self, p_keys_list, p_sep = EXTRA_SEP):
        """Return the tag values of all given keys
        separated by the given separator."""
        _list = []
        # add values of all keys in list
        for _key in p_keys_list:
            _value = self.get_tag(_key, p_sep)
            if _value:
                _list.append(_value)
        return string.join(_list, p_sep)

    def get_artist(self):
        """Return the artist(s)."""
        return self.__get_tags_in_one(ARTIST, NAMES_SEP)

    def get_title(self):
        """Return the title."""
        return self.__get_tags_in_one(TITLE)

    def get_album(self):
        """Return the album."""
        return self.__get_tags_in_one(ALBUM)

    def get_track(self):
        """Return the track number."""
        return self.__get_tags_in_one(TRACKNUMBER)

    def get_year(self):
        """Return the year."""
        return self.__get_tags_in_one(YEAR)

    def get_genre(self):
        """Return the genre."""
        return self.__get_tags_in_one(GENRE)

    def get_description(self):
        """Return the comment."""
        return self.__get_tags_in_one(DESCRIPTION)

    def to_song(self):
        """Return the corresponding song."""
        # create a disc
        _disc = Disc(self.get_album())
        # create an artist
        _artist = Artist('')
        _names = []
        for _key in ARTIST:
            if self.__tags.has_key(_key):
                for _name in self.__tags[_key]:
                    _names.append(_name)
        if len(_names) > 0:
            _artist = Artist(_names[0])
            for _name in _names[1:]:
                _artist.add_name(_name)
        # create a tune
        _tune = Tune(self.get_title(), self.get_year())
        # get the track
        _track = self.get_track()
        # get the genre
        _genre = self.get_genre()
        # get the comment
        _desc = self.get_description()
#        _song = Song(_song_singers, _tune, _disc, _track, _genre, _desc, _song_feat)
        _song = Song(_artist, _tune, _disc, _track, _genre, _desc)
        return _song

    def __get_tag_string(self, p_keys_list):
        """Return a tag assignement, ie tag=value\n."""
        _ret = ''
        # for each key
        for _key in p_keys_list:
            if self.__tags.has_key(_key):
                # for each data corresponding to the key
                _data_list = self.__tags[_key]
                for _data in _data_list:
                    # if the data is not empty, add a line
                    if _data:
                        _ret += p_keys_list[0] + "=" + _data + "\n"
        return _ret

    def __str__(self):
        "Return a representation as a string."
        _ret = ""
        _ret += self.__get_tag_string(TITLE)
        _ret += self.__get_tag_string(ARTIST)
        _ret += self.__get_tag_string(ALBUM)
        _ret += self.__get_tag_string(TRACKNUMBER)
        _ret += self.__get_tag_string(YEAR)
        _ret += self.__get_tag_string(GENRE)
        _ret += self.__get_tag_string(DESCRIPTION)
        return _ret


if __name__ == "__main__":
    from Song import get_complete_song
    
    def print_tags_set(p_tags_set) :
        print "Artist :", _set.get_artist()
        print "Title :", _set.get_title()
        print "Album :", _set.get_album()
        print "Track :", _set.get_track()
        print "Year :", _set.get_year()
        print "Genre :", _set.get_genre()
        print "Description :", _set.get_description()
        print "String :\n", str(_set)
        print "Song : ", _set.to_song()
        
    print 'Test of TagsSet.py'
    print 'Construct a TagsSet from scratch'
    _set = TagsSet()
    _set.add_tag('Artist', 'Julien')
    _set.add_tag('ARTIST', 'Ju|es')
    _set.add_tag('title', 'My song')
    _set.add_tag('ALBUM', 'My album')
    _set.add_tag('disc', 'My first disc')
    _set.add_tag('track', '7')
    _set.add_tag('DATE', '10/01/1981')
    _set.add_tag('GENRE', 'Groove')
    _set.add_tag('Comment', 'My first song')
    print_tags_set(_set)
    print 'Construct a TagsSet from a song'
    _song = get_complete_song('Ju|es', 'My title', 'My disc')
    _song.get_artist().add_name('Julien')
    _set = TagsSet(_song)
    print_tags_set(_set)

