# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import DirectoryNotWritableException
from util.File import FileNotFoundException
from util.File import FileAlreadyExistsException
from util.File import FileNotReadableException
from util.File import File

from MusicFile import FileNotOggVorbisException
from TagsSet import TagsSet
from Task import Task
from Process import is_in_path
from Process import ToolNotFoundException
from Process import Process

TAGS_WRITING_TOOL = "vorbiscomment"

TAGS_WRITING_TOOL_EXISTS = is_in_path(TAGS_WRITING_TOOL)


class TagsWriting(Task):
    """A task which writes tags of an ogg Vorbis file."""

    __src_file_path = ""

    __dest_file_path = ""

    __tags_set = None

    __process = None

    def __init__(self, p_src_path, p_dest_path, p_tags_set):
        Task.__init__(self)        
        self.__src_file_path = p_src_path
        self.__dest_file_path = p_dest_path
        self.__tags_set = p_tags_set
        self.__process = None

    def get_src_file_path(self):
        return self.__src_file_path

    def get_dest_file_path(self):
        return self.__dest_file_path

    def get_tags_set(self):
        return self.__tags_set


    def start(self):
        """Write an Ogg Vorbis file with tags passed in the standard input.

        Raise ToolNotFoundException if the tool for writing tags is not available.
        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable.
        Raise FileNotOggVorbisFileException if the file is not an Ogg Vorbis file.
        Raise DirectoryNotWritableException if the directory cannot be
        created in the root directory or if the file cannot be copied in the
        directory.
        Raise ProcessException."""
        if not TAGS_WRITING_TOOL_EXISTS:
            raise ToolNotFoundException, TAGS_WRITING_TOOL
        _file = File(self.__src_file_path)
        # verify that the file exists
        if not _file.exists():
            raise FileNotFoundException, _file.get_path()
        # verify that the file is readable
        if not _file.is_readable():
            raise FileNotReadableException, _file.get_path()
        # verify that the file is an Ogg Vorbis file
        if not _file.has_ext('.ogg'):
            raise FileNotOggVorbisException, _file.get_path()
        # get correct directory and file name
        _file = File(self.__dest_file_path)
        # verify that the file does not exist
        if _file.exists():
            raise FileAlreadyExistsException, _file.get_path()
        # get the destination directory and create it if necessary
        _dir = _file.get_parent_directory()
        if not _dir.exists():
            _dir.create()
        # verify that the destination directory is writable
        if not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        # launch process        
        _cmd = _cmd = TAGS_WRITING_TOOL + " -w \"" + self.__src_file_path + "\" \"" + self.__dest_file_path + "\""
#        print 'Command :', _cmd
#        _process = self.launch_process(_cmd)
        self.__process = Process(_cmd, Process.IO_LOG_FILE)
        # write the set of tags
#        print 'Tags :'
#        print str(self.__tags_set)
        self.__process.write_and_close(str(self.__tags_set))
        self.__process.wait()

    def stop(self):
        if self.__process is not None:
            self.__process.kill()


if __name__ == '__main__':
    import sys
    from Song import get_complete_song
    if not TAGS_WRITING_TOOL_EXISTS:
        print 'Required tool : %s' % TAGS_WRITING_TOOL
    elif len(sys.argv) != 6:
        print 'Usage : %s %s src_file dest_file artist title album' % (sys.executable, sys.argv[0])
    else:
        _song = get_complete_song(sys.argv[3], sys.argv[4], sys.argv[5])
        _set = TagsSet(_song)
        _task = TagsWriting(sys.argv[1], sys.argv[2], _set)
        print 'Writing the file :'
        print '\t%s' % _task.get_src_file_path()
        print 'to the file :'
        print '\t%s' % _task.get_dest_file_path()
        print 'with tags  :'
        print '%s' % _task.get_tags_set()
        print 'Please wait...'
        _task.start()

