# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import Directory
from util.File import File

from Task import Task
from Encoding import ENCODING_TOOL
from Encoding import ENCODING_TOOL_EXISTS
from Encoding import Encoding
from SongAdding import get_song_adding_tools
from SongAdding import can_add_song
from SongAdding import SongAdding


def get_wave_importing_tools():
    return [ENCODING_TOOL] + get_song_adding_tools()


def can_import_wave():
    return (ENCODING_TOOL_EXISTS and can_add_song())


class WaveImporting(Task):
    """A task which encodes a wave file and imports the encoded file
    into the library."""

    __encoding = None

    __song_adding = None

    def __init__(self, p_manager, p_library, p_src_file_path, p_song):
        Task.__init__(self)
        # get a temporary file path
        _dir = Directory(p_manager.get_tmp_path())
        _inode = _dir.get_inode('song.ogg')
        # create an encoding task
        self.__encoding = Encoding(p_src_file_path, _inode.get_path())
        # create a song adding task
        self.__song_adding = SongAdding(p_manager, p_library, _inode.get_path(), p_song)

    def get_src_file_path(self):
        return self.__encoding.get_src_file_path()

    def get_dest_file_path(self):
        return self.__song_adding.get_dest_file_path()

    def get_song(self):
        return self.__song_adding.get_song()

    def start(self):
        """Encode a wave file and add the result file into the library.

        Raise ToolNotFoundException if a tool is not available.
        Raise FileNotFoundException if a file does not exist.
        Raise FileNotReadableException if a file is not readable.
        Raise FileNotOggVorbisFileException if a file is not an Ogg Vorbis file.
        Raise DirectoryNotWritableException if a directory cannot be
        created in the root directory or if the file cannot be copied in the
        directory.
        Raise ProcessException if a process stops irregularly."""
        # encode the wave file
        self.__encoding.start()
        # add the Ogg Vorbis-encoded song to the library
        self.__song_adding.start()
        # remove the temporary file
        _file = File(self.__song_adding.get_src_file_path())
        _file.remove()

    def stop(self):
        Task.stop(self)
        if self.__encoding is not None:
            self.__encoding.stop()
        if self.__song_adding is not None:
            self.__song_adding.stop()
        

if __name__ == '__main__':
    import sys
    import string
    from Manager import Manager
    from Library import Library
    from Song import get_complete_song
    if not can_import_wave():
        print 'Required tools : %s' % string.join(get_wave_importing_tools())
    elif len(sys.argv) != 5:
        print 'Usage : %s %s file artist title album' % (sys.executable, sys.argv[0])
    else:
        _manager = Manager()
        _lib = Library()
        _file_path = sys.argv[1]
        _song = get_complete_song(sys.argv[2], sys.argv[3], sys.argv[4])
        _task = WaveImporting(_manager, _lib, _file_path, _song)
        print 'Importing the file :'
        print '\t%s' % _task.get_src_file_path()
        print 'to the file :'
        print '\t%s' % _task.get_dest_file_path()
        print 'with song :'
        print '\t%s' % _task.get_song()
        print 'Please wait...'
        _task.start()
        print _lib

