# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager

from Session import Session
from Menu import Menu

class MainWindow(gtk.GtkWindow):
    """A main window with a pages notebook."""

    __notebook = None

    __pages = []

    __status_label = None

    def __init__(self):
        # crzate the window
        gtk.GtkWindow.__init__(self, gtk.WINDOW_TOPLEVEL)
        self.__pages = []
        self.set_title('Maestro')
        self.connect('key_press_event', self.__window_key_pressed)
        self.connect('destroy', self.__destroy)
        self.set_usize(600, 400)
        # create a vertical box
        _box = gtk.GtkVBox()
        # create a notebook and add it to the box
        self.__notebook = gtk.GtkNotebook()
        self.__notebook.connect_after('button_press_event', self.__note_book_clicked)
        self.__notebook.connect_after('switch_page', self.__note_book_switched)
        self.__notebook.show()
        _box.pack_start(self.__notebook, gtk.TRUE)
        # create a status label and add it to the box
        self.__status_label = gtk.GtkLabel()
        self.__status_label.show()
        _box.pack_start(self.__status_label, gtk.FALSE)
        # show the box and add it to the window
        _box.show()
        self.add(_box)

    def set_status(self, p_message):
        """Change the status message."""
        self.__status_label.set_text(p_message)
        

    def get_current_page(self):
        """Return the page which is currently showed."""
        _page_num = self.__notebook.get_current_page()
        return self.__pages[_page_num]

    def add_tab(self, p_page):
        """Add the given page."""
        p_page.show()
        _label = gtk.GtkLabel(p_page.get_title())
        _label.show()
        self.__notebook.append_page(p_page, _label)
        self.__pages.append(p_page)
        self.__notebook.set_page(-1)

    def update_labels(self):
        """Update all labels of the notebook."""
        for _page in self.__pages:
            self.__notebook.set_tab_label_text(_page, _page.get_title())
            
    def close_page(self, p_page_num):
        """Close the page at the given index."""
        _page = self.__pages[p_page_num]
        if _page.is_closable():
            _page.hide()
            self.__notebook.remove_page(p_page_num)
            del self.__pages[p_page_num]

    def close_current_page(self):
        """Close the page which is currently showed."""
        _page_num = self.__notebook.get_current_page()
        self.close_page(_page_num)

    def get_page(self, p_page_num):
        """Return the page at the given index."""
        return self.__pages[p_page_num]

    def get_pages_number(self):
        """Return the number of pages."""
        return len(self.__pages)

    def quit(self):
        """Save the play list (if needed), the library (if needed),
        the path configuration and quit."""
        _manager = Manager()
        # save play lists if needed
        _session = Session()
        if _session.play_lists_were_modified():
#            print "Saving play lists..."
            _task = _manager.get_play_lists_saving()
            _manager.get_tasks_thread().add_task(_task)
        # save the library if needed
        if _session.library_was_modified():
#            print "Saving library..."
            _task = _manager.get_library_saving()
            _manager.get_tasks_thread().add_task(_task)
        # save the path configuration
#        print "Saving path configuration..."
        _task = _manager.get_path_config_saving()
        _manager.get_tasks_thread().add_task(_task)
        gtk.idle_add(self.__quit)


    def __quit(self):
        _manager = Manager()
        # wait for the end of all tasks
        _manager.get_tasks_thread().join()
        # quit the application
        gtk.mainquit()

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __destroy(self, *args):
#        print '__destroy', self
        self.quit()

    def __note_book_clicked(self, p_notebook, p_event):
#        print '__note_book_clicked', self, p_notebook, p_event
        if p_event.button == 3:
            _page_num = p_notebook.get_current_page()
            if _page_num >= 0 and _page_num < self.get_pages_number():
                _page = self.__pages[_page_num]
                _menu = _page.get_menu()
                if _menu is None:
                    _menu = Menu()
                else:
                    if _page.is_closable():
                        _menu.append_separator()
                if _page.is_closable():
                    _menu.append_item('Close', self.__close, _page_num)
                _menu.show()
                _menu.popup(None, None, None, p_event.button, p_event.time)


    def __close(self, p_menu_item, p_page_num):
#        print '__close', self, p_menu_item, p_page_num
        self.close_page(p_page_num)
        
    def __note_book_switched(self, p_notebook, p_page, p_page_num):
#        print '__note_book_switched', self, p_notebook, p_page, p_page_num
        if p_page_num >= 0 and p_page_num < len(self.__pages):
            _page = self.__pages[p_page_num]
            self.__notebook.set_tab_label_text(_page, _page.get_title())
            _page.update()
            self.set_status(_page.get_status())

    def __window_key_pressed(self, p_window, p_event):
#        print '__window_key_pressed', self, p_window, p_event
        if p_event.keyval >= 0 and p_event.keyval <= 256:
            _key = chr(p_event.keyval)
        else:
            _key = ''
        if (p_event.state & gtk.GDK.CONTROL_MASK):
            if _key == 'q':
                self.quit()
            if _key == 'w':
                if self.__notebook.get_current_page() < self.get_pages_number() - 1:
                    self.__notebook.next_page()
                else:
                    self.__notebook.set_page(0)
            if _key == 'W':
                if self.__notebook.get_current_page() > 0:
                    self.__notebook.prev_page()
                else:
                    self.__notebook.set_page(-1)
