# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager
from base.Selection import Selection
from base.PlayList import PlayList
from base.SongEditing import get_song_editing_tools
from base.SongEditing import can_edit_song

from List import List
from PlayListsSetMenu import PlayListsSetMenu
from Menu import Menu
from SearchPage import SearchPage
from SongEditingPage import SongEditingPage
from SongRemovingPage import SongRemovingPage
#from PlayListPage import PlayListPage
from MissingToolsPage import MissingToolsPage
from Session import Session

class SelectionList(List):
    """A list that displays a selection of songs."""

    __editable = 1

    def __init__(self, p_selection, p_editable = 1):
        List.__init__(self, p_selection, 6, ['Title', 'Artist', 'Disc', 'Year', 'Track', 'Genre'])
        self.set_widths([150, 100, 150, 30, 30])
        self.__editable = p_editable

#    def update(self):
#        """Update the list."""
#        List.update(self)
#        # update the status message
#        _session = Session()
#        _session.get_main_window().set_status(self.get_status())
        
    ###########################################################################
    # ACCESSORS
    ###########################################################################

    def get_tuple(self, p_song_file):
        """Return the tuple corresponding to the given song file."""
        _song = p_song_file.get_song()
        if _song is not None:
            _title = _song.get_tune().get_title()
            # if song has no real title, the path become is title for displaying
            if not _title.strip():
                _title = p_song_file.get_path()
            _tuple = (_title, str(_song.get_artist()), str(_song.get_disc()),
                      _song.get_tune().get_year(), _song.get_track(), _song.get_genre())
        else:
            _tuple = (p_song_file.get_path(), '', '', '', '', '')
        return _tuple

    def get_selection(self):
        """Return the selection."""
        return self.get_sequence()

    def get_selected_songs(self):
        """Return the selection of all selected songs."""
        _items = self.get_selected_items()
        _ret = Selection(_items)
        return _ret

    def __append_common_items(self, p_menu, p_selection):
        """Append to the given menu, some menu items relative
        to the given selection."""
        _menu = p_menu
        _menu.append_item('Append to play queue', self.__append_to_pq, p_selection)
        _sub_menu = PlayListsSetMenu(self.__append_to_pl, p_selection)
        _menu.append_menu('Append to play list', _sub_menu)
        _menu.append_item('Search...', self.__search, p_selection)
        _menu.append_item('Create play list', self.__create, p_selection)
        return _menu

    def append_menu(self, p_menu = None):
        """Add global menu items to the given menu

        Create a new menu if the given menu is None."""
        _menu = p_menu
        if _menu is None:
            _menu = Menu()
#            _menu = SimpleSelectionMenu(self.__selection)  # A INTEGRER ICI
        _menu = self.__append_common_items(_menu, self.get_selection())
        _menu.append_separator()
        if self.__editable:
            # create the sort menu
            _sub_menu = Menu()
            _sub_menu.append_item('Title', self.__sort_by_title)
            _sub_menu.append_item('Year', self.__sort_by_year)
            _sub_menu.append_item('Artist', self.__sort_by_artist)
            _sub_menu.append_item('Disc', self.__sort_by_disc)
            _sub_menu.append_item('Track', self.__sort_by_track)
            _sub_menu.append_item('Genre', self.__sort_by_genre)
            # append the new menu
            _menu.append_menu('Sort by', _sub_menu)
            # append the randomize menu item
            _menu.append_item('Randomize', self.__randomize)
            # append the clear menu item
            _menu.append_item('Clear', self.__clear)
            # append a separator and all selection items
            _menu.append_separator()
        _menu = List.append_menu(self, _menu)
        return _menu

    def get_menu(self):
        """Return the selection popup menu.

        It is a convenience way for derived class to redfine this method."""
        _sel = self.get_selected_songs()
        _menu = Menu()
        _menu = self.__append_common_items(_menu, _sel)
        if self.__editable:
            _menu.append_item('Remove', self.__remove)
        _menu.append_separator()
        _menu.append_item('Edit...', self.__edit)
        _menu.append_item('Remove from all...', self.__remove_from_all)
        return _menu

    def get_status(self):
        """Return the status message."""
        _ret = '%d songs - %d selected songs' % (self.get_selection().get_length(), self.get_selected_songs().get_length())
        return _ret

    ###########################################################################
    # MODIFICATORS
    ###########################################################################

    def set_selection(self, p_selection):
        """Set the given selection.

        Warning : the update method must be called to display
        the new selection."""
        self.set_sequence(p_selection)

    def sort_by_title(self):
        """Sort the list by title."""
        self.get_selection().sort_by_title()
        self.update()
        
    def sort_by_year(self):
        """Sort the list by year."""
        self.get_selection().sort_by_year()
        self.update()
        
    def sort_by_artist(self):
        """Sort the list by artist."""
        self.get_selection().sort_by_artist()
        self.update()
        
    def sort_by_disc(self):
        """Sort the list by disc."""
        self.get_selection().sort_by_disc()
        self.update()
        
    def sort_by_track(self):
        """Sort the list by track."""
        self.get_selection().sort_by_track()
        self.update()
        
    def sort_by_genre(self):
        """Sort the list by genre."""
        self.get_selection().sort_by_genre()
        self.update()
        
    def randomize(self):
        """Randomize the list."""
        self.get_selection().random()
        self.update()

    def clear(self):
        """Clear the list."""
        self.get_selection().clear()
        # update the list
        self.update()
        # update the status message
        _session = Session()
        _session.get_main_window().set_status(self.get_status())

    def remove(self, p_rows):
        """Remove the given rows from the list."""
        # unselect all rows before delete any rows
        self.unselect_all()
        # remove all songs begining from the end
        _sel = self.get_selection()
        p_rows.sort()
        p_rows.reverse()
        for _row in p_rows:
            _sel.delete(_row)
        # update the list
        self.set_selection(_sel)
        self.update()
        # update the status message
        _session = Session()
        _session.get_main_window().set_status(self.get_status())

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __append_to_pq(self, p_menu_item, p_selection):
        """Append the given selection to the play queue."""
#        print '__append_to_pq', self, p_menu_item
        _manager = Manager()
        _queue = _manager.get_play_thread().get_queue()
        _queue.extend(p_selection)
        if _queue is self.get_selection():
            # the play queue and the list selection is the same object,
            # ie, the selection is the play queue and it has been modified
            self.update()

    def __append_to_pl(self, p_play_list, p_selection):
        """Append the given selection to the given play list
        and set play lists as modified."""
#        print '__append_to_pl', self, p_play_list
        p_play_list.extend(p_selection)
        _session = Session()
        _session.set_play_lists_as_modified()
        if p_play_list is self.get_selection():
            # the play list and the list selection is the same object,
            # ie, the selection is a play list and it has been modified
            self.update()

    def __search(self, p_menu_item, p_selection):
        """Open the search page to look for songs among the given selection."""
#        print '__search', self, p_menu_item, p_selection
        _page = SearchPage(p_selection)
        _session = Session()
        _session.get_main_window().add_tab(_page)

    def __create(self, p_menu_item, p_selection):
        """Create a new play list with the given selection,
        set play lists as modified
        and open a page for managing it."""
#        print '__create :', self, p_menu_item, p_selection
        # create a new play list
        _pl = PlayList('Play list', p_selection)
        # add the play list to the manager
        _manager = Manager()
        _manager.get_play_lists_set().append(_pl)
        _session = Session()
        _session.set_play_lists_as_modified()
        # create a page and display it
        from PlayListPage import PlayListPage
        _page = PlayListPage(_pl)
        _session.get_main_window().add_tab(_page)

    def __sort_by_title(self, p_menu_item):
        """Sort the list by title."""
#        print '__sort_by_title', self, p_menu_item
        self.sort_by_title()
        
    def __sort_by_year(self, p_menu_item):
        """Sort the list by year."""
#        print '__sort_by_year', self, p_menu_item
        self.sort_by_year()
        
    def __sort_by_artist(self, p_menu_item):
        """Sort the list by artist."""
#        print '__sort_by_artist', self, p_menu_item
        self.sort_by_artist()
        
    def __sort_by_disc(self, p_menu_item):
        """Sort the list by disc."""
#        print '__sort_by_disc', self, p_menu_item
        self.sort_by_disc()
        
    def __sort_by_track(self, p_menu_item):
        """Sort the list by track."""
#        print '__sort_by_track', self, p_menu_item
        self.sort_by_track()
        
    def __sort_by_genre(self, p_menu_item):
        """Sort the list by genre."""
#        print '__sort_by_title', self, p_menu_item
        self.sort_by_genre()
        
    def __randomize(self, p_menu_item):
        """Randomize the list."""
#        print '__randomize', self, p_menu_item
        self.randomize()

    def __clear(self, p_menu_item):
        """Clear the list."""
#        print '__clear', self, p_menu_item
        self.clear()

    def __remove(self, p_menu_item):
        """Remove the selected songs from the selection."""
#        print '__remove', self, p_menu_item
        # get selected songs
        _rows = self.get_selected_rows()
        # unselect all rows before delete any rows
        self.remove(_rows)

    def __edit(self, p_menu_item):
        """Open a page for editing the selected songs."""
#        print '__edit', self, p_menu_item
        if not can_edit_song():
            _msg = 'Sorry but you cannot edit an Ogg Vorbis file.'
            _tools = get_song_editing_tools()
            _mtpage = MissingToolsPage(_msg, _tools)
            _session = Session()
            _session.get_main_window().add_tab(_mtpage)
        else:
            _sel = self.get_selected_songs()
            if _sel.get_length() > 0:
                _page = SongEditingPage(_sel)
                _session = Session()
                _session.get_main_window().add_tab(_page)


    def __remove_from_all(self, p_menu_item):
        """Open a page for removing the selected songs from the library."""
#        print '__remove_from_all', self, p_menu_item
        _sel = self.get_selected_songs()
        if _sel.get_length() > 0:
            _page = SongRemovingPage(_sel)
            _session = Session()
            _session.get_main_window().add_tab(_page)

