# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import os.path

from Inode import InodeException
from Inode import Inode


class DirectoryException(InodeException):
    """A base exception for a directory."""

    pass


class DirectoryNotFoundException(DirectoryException):
    """An exception raised when a directory does not exist."""

    pass


class DirectoryNotEmptyException(DirectoryException):
    """An exception raised when a directory is not empty."""

    pass


class DirectoryAlreadyExistsException(DirectoryException):
    """An exception raised when a directory already exists."""

    pass


class DirectoryNotReadableException(DirectoryException):
    """An exception raised when a file or a directory cannot be created in a directory."""

    pass


class DirectoryNotWritableException(DirectoryException):
    """An exception raised when a file or a directory cannot be created in a directory."""

    pass


class DirectoryNotAccessibleException(DirectoryException):
    """An exception raised when a directory cannot be accessible."""

    pass


class Directory(Inode):
    """A directory."""

    def exists(self):
        """Determine if the directory exists and is a directory."""
        return self.is_directory()

    def get_parent(self):
        """Return the parent directory."""
        _inode = Inode.get_parent(self)
        _dir = Directory(_inode.get_path())
        return _dir

    def get_inode(self, p_name):
        """Return an inode with its complete path."""
        _path = os.path.join(self.get_path(), p_name)
        _ret = Inode(_path)
        return _ret

    def get_file_path(self, p_name):
        """Return a file with its complete path."""
        _ret = os.path.join(self.get_path(), p_name)
        return _ret

    def get_directory(self, p_name):
        """Return a directory with its complete path."""
        _path = self.join_path(p_name)
        _ret = Directory(_path)
        return _ret

    def is_empty(self):
        """Determine if the directory contains no file and no directory.

        Raise DirectoryNotFoundException if the directory does not exist.
        Raise DirectoryNotReadableException if the directory cannot be read."""
        if not self.exists():
            raise DirectoryNotFoundException, self.get_path()
        if not self.is_readable():
            raise DirectoryNotReadableException, self.get_path()
        _file_names = os.listdir(self.get_path())
        return (_file_names == [])

    def list_names(self):
        """Return a list of files and directories.

        Raise DirectoryNotFoundException if the directory does not exist.
        Raise DirectoryNotReadableException if the directory cannot be read."""
        if not self.exists():
            raise DirectoryNotFoundException, self.get_path()
        if not self.is_readable():
            raise DirectoryNotReadableException, self.get_path()
        _entries = os.listdir(self.get_path())
        return _entries

    def list(self):
        """Return a list of files and directories.

        Raise DirectoryNotFoundException if the directory does not exist.
        Raise DirectoryNotReadableException if the directory cannot be read."""
        _ret = []
        _entries = self.list_names()
        for _name in _entries:
            _inode = self.get_inode(_name)
            _ret.append(_inode)
        return _ret

    def list_files(self):
        """Return a list of files.

        Raise DirectoryNotFoundException if the directory does not exist.
        Raise DirectoryNotReadableException if the directory cannot be read."""
        _ret = self.list()
        _ret = filter(Inode.is_file, _ret)
        return _ret

    def change(self):
        """Change the current directory.

        Raise DirectoryNotFoundException if the directory does not exist.
        Raise DirectoryNotAccessibleException if the directory cannot be accessed."""
        if not self.exists():
            raise DirectoryNotFoundException, self.get_path()
        if not self.is_accessible():
            raise DirectoryNotAccessibleException, self.get_path()
        os.chdir(self.get_path())

    def create(self):
        """Create a directory and its parents recursively if needed.

        Raise DirectoryAlreadyExistsException if the directory already exists.
        Raise DirectoryNotWritableException if a parent directory cannot be written."""
        if self.exists():
            raise DirectoryAlreadyExistsException, self.get_path()
        _dir = self.get_parent()
        try:                
            _dir.create()
        except DirectoryAlreadyExistsException:
            pass
        if not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        os.mkdir(self.get_path())
    
    def delete(self):
        """Delete a directory and its parents recursively if empty.

        Raise DirectoryNotFoundException if the directory does not exist.
        Raise DirectoryNotEmptyException if the directory is not empty.
        Raise DirectoryNotWritableException if a parent directory cannot be written."""
        # verify that the directory exists
        if not self.exists():
            raise DirectoryNotFoundException, self.get_path()
        # verify that the directory is empty
        if not self.is_empty():
            raise DirectoryNotEmptyException, self.get_path()
        # verify that the parent diretory can be written
        _dir = self.get_parent()
        if not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        # remove the directory
        os.rmdir(self.get_path())
        try:
            _dir.delete()
        except DirectoryException:
            pass
    
if __name__ == "__main__":
    _dir = Directory("/tmp/music/Louise_Attaque/La_plume/")
    print "Directory :", _dir
    print "Path :", _dir.get_path()
    print "Exists :", _dir.exists()
    print "Parent :", _dir.get_parent()
    print "Creating"
    _dir.create()
    print "Exists :", _dir.exists()
    print "Readable :", _dir.is_readable()
    print "Writable :", _dir.is_writable()
    print "Accessible :", _dir.is_accessible()
    print "changing"
    _dir.change()
    
