# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import os.path
import shutil
import re

from Inode import InodeException
from Inode import Inode
from Directory import DirectoryNotFoundException
from Directory import DirectoryNotReadableException
from Directory import DirectoryNotWritableException
from Directory import DirectoryNotAccessibleException
from Directory import DirectoryException
from Directory import Directory
from String import String


class FileException(InodeException):
    """A base exception for a file."""

    pass


class FileNotFoundException(FileException):
    """An exception raised when a file does not exist."""

    pass


class FileAlreadyExistsException(FileException):
    """An exception raised when a file already exists."""

    pass


class FileNotReadableException(FileException):
    """An exception raised when a file is not readable."""

    pass


class FileNotWritableException(FileException):
    """An exception raised when a file is not writable."""

    pass


class File(Inode):
    """A file."""

    def has_ext(self, p_ext):
        """Determine if the file path has the given extension.

        p_ext -- extension that begins with a dot
        """
        _split = os.path.splitext(self.get_path())
        _ext1 = String(_split[1])
        _ext2 = String(p_ext)
        return _ext1 == _ext2

    def exists(self):
        """Determine if the file exists and is a file."""
        return self.is_file()

    def get_parent_directory(self):
        """Return the parent directory."""
        _dir = Directory(os.path.dirname(self.get_path()))
        return _dir

    def open_to_read(self):
        """Open the file path to read it and return a descriptor.

        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable."""
        if not self.exists():
            raise FileNotFoundException, self.get_path()
        if not self.is_readable():
            raise FileNotReadableException, self.get_path()
        _f = open(self.get_path(), 'r')
        return _f        

    def open_to_write(self):
        """Open the file path to write it and return a descriptor.

        Raise DirectoryNotWritableException if the file does not exist and
        the parent directory is not writable.
        Raise FileNotWritableException if the file exists and is not
        writable."""
        _dir = self.get_parent_directory()
        if not self.exists() and not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        if self.exists() and not self.is_writable():
            raise FileNotWritableException, self.get_path()
        _f = open(self.get_path(), 'w')
        return _f        

    """
    def complete(self):
        "Return an existing path which could correspond to the path.

        Return the file path if no files match it."
        _ret = self.get_path()
        if self.is_directory():
            if _ret[-1] != os.sep:
                _ret += os.sep
        else:
            _dir = self.get_parent_directory()
            _name = self.get_name()
            try:
                _names = _dir.list_names()
            except DirectoryException:
                pass
            else:
                _len = len(_names)
                if _len > 0:
                    _prog = re.compile(_name)
                    _index = 0
                    while _index < _len and not _prog.match(_names[_index]):
                        _index += 1
                    if _index < _len:
                        _ret = _dir.get_file_path(_names[_index])
        return _ret
    """

    def complete(self, p_only_dir):
        """Return a list of existing paths which could correspond
        to the path.

        Return the list with the file path if no files match it."""
        _ret = []
        _path = self.get_path()
        if self.is_directory():
#            if _path[-1] != os.sep:
#                _ret = [_path]
#            else:
                _dir = Directory(_path)
                _names = _dir.list_names()
                for _cur_name in _names:
                    _ret.append(_dir.get_file_path(_cur_name))
        else:
            _dir = self.get_parent_directory()
            _name = self.get_name()
            try:
                _names = _dir.list_names()
            except DirectoryException:
                pass
            else:
                _prog = re.compile(_name)
                for _cur_name in _names:
                    if _prog.match(_cur_name):
                        _ret.append(_dir.get_file_path(_cur_name))                    
                """
                _len = len(_names)
                if _len > 0:
                    _prog = re.compile(_name)
                    _index = 0
                    while _index < _len and not _prog.match(_names[_index]):
                        _index += 1
                    if _index < _len:
                        _ret = _dir.get_file_path(_names[_index])
                """
        return _ret


    def copy(self, p_dest_file):
        """Copy from the file as source to the given destination file.

        Raise FileNotFoundException if the source file does not exist.
        Raise FileNotReadableException if the source file cannot be read.
        Raise DirectoryNotFoundException if the destination directory does not exist.
        Raise DirectoryNotWritableException if the destination directory cannot be written."""
        # verify if the file exists
        if not self.exists():
            raise FileNotFoundException, self.get_path()
        # verify if the file can be read
        if not self.is_readable():
            raise FileNotReadableException, self.get_path()
        # get the parent directory of the destination file
        _dir = p_dest_file.get_parent_directory()
        # verify if the destination directory exists
        if not _dir.exists():
            raise DirectoryNotFoundException, _dir.get_path()
        # verify if the destination directory is writable
        if not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        # copy the source file to the destination file
        shutil.copy(self.get_path(), p_dest_file.get_path())

    def rename(self, p_dest_file):
        """Rename the file as the given destination file.

        Raise FileNotFoundException if the source file does not exist.
        Raise FileNotReadableException if the source file cannot be read.
        Raise DirectoryNotFoundException if the destination directory does not exist.
        Raise DirectoryNotWritableException if the source directory or the destination
        directory cannot be written."""
        # verify if the file exists
        if not self.exists():
            raise FileNotFoundException, self.get_path()
        # verify if the file can be read
        if not self.is_readable():
            raise FileNotReadableException, self.get_path()
        # get the parent directory of the source file
        _dir = self.get_parent_directory()
        # verify if the directory is writable
        if not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        # get the parent directory of the destination file
        _dir = p_dest_file.get_parent_directory()
        # verify if the directory exists
        if not _dir.exists():
            raise DirectoryNotFoundException, _dir.get_path()
        # verify if the directory is writable
        if not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        # move the source file to the destination file
        os.rename(self.get_path(), p_dest_file.get_path())

    def remove(self):
        """Remove the file.

        Raise FileNotFoundException if the source file does not exist.
        Raise DirectoryNotWritableException if the destination directory cannot be written."""
        # verify if the file exists
        if not self.exists():
            raise FileNotFoundException, self.get_path()
        # get the parent directory of the destination file
        _dir = self.get_parent_directory()
        # verify if the destination directory is writable
        if not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        # remove the file
        os.unlink(self.get_path())

    def move(self, p_dest_file):
        """Move the file to the detination file.
        
        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file cannot be read.
        Raise DirectoryNotFoundException if the destination directory does not exist.
        Raise DirectoryNotWritableException if the file or a directory cannot
        be removable or if the temporary file cannot be created."""
        self.copy(p_dest_file)
        self.remove()

if __name__ == '__main__':
    _path = '/us'
    _file = File(_path)
    print _file
    _path = _file.complete()
    _file = File(_path + '/bi')
    print _file
    _path = _file.complete()
    _file = File(_path + '/pyt')
    print _file
    _path = _file.complete()
    _file = File(_path)
    print _file
    
        
