# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import os.path
import re

from UserException import UserException


class InodeException(UserException):
    """A base class for all exceptions related to a file system."""

    pass


class Inode:
    """An inode."""

    __path = ""

    def __init__(self, p_path):
        """Create the inode."""
        self.__path = os.path.normpath(p_path)

    def get_path(self):
        """Return the path of the inode."""
        return self.__path

    def has_path(self, p_path):
        """Determine if the file has the given path."""
        return self.__path == p_path

    def get_name(self):
        """Return the name of the inode."""
        return os.path.basename(self.__path)

    def get_parent(self):
        """Return the parent inode."""
        _inode = Inode(os.path.dirname(self.get_path()))
        return _inode

    def join_path(self, p_path):
        """Join the path with the given path."""
        return os.path.join(self.get_path(), p_path)

    def exists(self):
        """Determine if the inode exists."""
        return os.access(self.__path, os.F_OK)

    def is_readable(self):
        """Determine if an inode is readable."""
        return os.access(self.__path, os.R_OK)

    def is_writable(self):
        """Determine if an inode is writable."""
        return os.access(self.__path, os.W_OK)

    def is_accessible(self):
        """Determine if an inode can be accessed."""
        return os.access(self.__path, os.X_OK)

    def is_executable(self):
        """Determine if an inode can be executed."""
        return os.access(self.__path, os.X_OK)

    def is_file(self):
        """Determine if the inode exists and is a file."""
        return os.path.isfile(self.get_path())

    def is_directory(self):
        """Determine if the inode exists and is a directory."""
        return os.path.isdir(self.get_path())

    def complete(self, p_only_dir):
        """Return a list of existing paths which could correspond
        to the path.

        Return the list with the file path if no files match it."""
        _ret = []
        if self.is_directory():
            # just list the directory
            try:
                _names = os.listdir(self.get_path())
            except IOError:
                pass
            else:
                for _cur_name in _names:
                    _ret.append(self.join_path(_cur_name))
        else:
            _inode = self.get_parent()
            _name = self.get_name()
            try:
                _names = os.listdir(_inode.get_path())
            except IOError:
                pass
            else:
                _prog = re.compile(_name)
                for _cur_name in _names:
                    if _prog.match(_cur_name):
                        _ret.append(_inode.join_path(_cur_name))
        # filter other that is not a directory if needed
        if p_only_dir:
            _ret = filter(os.path.isdir, _ret)
        return _ret


    def __cmp__(self, p_inode):
        """Compare with another inode."""
        _ret = 1
        if p_inode is not None:
            _ret = cmp(self.__path, p_inode.__path)
        return _ret

    def __str__(self):
        """Return the path."""
        return self.__path
    
