# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import threading

from SyncList import SyncList

class QueuedThread(threading.Thread):
    "A thread that try to empty a queue by handling all items."

    __queue = None

    __event = None

    __current_item = None

    __callback = None

    __lock = None

    __kill = 0

    def __init__(self, p_name, p_queue = None):
        threading.Thread.__init__(self, name = p_name)
        if p_queue is None:
            self.__queue = SyncList()
        else:
            self.__queue = p_queue
        self.__current_item = None
        self.__callback = None
        self.__lock = threading.Lock()
        self.__event = threading.Event()
        self.__kill = 0

    ###########################################################################
    # ACCESSORS
    ###########################################################################

    def is_alive(self):
        """Determine if the thread is still running."""
        return self.isAlive()

    def get_queue(self):
        """Return the queue."""
        return self.__queue

    def get_current_item(self):
        """Return the item which is currently handled.

        Return None if no item is currently handled"""
        self.__lock.acquire()
        _ret = self.__current_item
        self.__lock.release()
        return _ret

    def get_changed_callback(self):
        """Get the function which must be called between two tasks."""
        self.__lock.acquire()
        _ret = self.__callback
        self.__lock.release()
        return _ret

    ###########################################################################
    # MODIFICATORS
    ###########################################################################

    def set_changed_callback(self, p_callback):
        """Set the function which must be called between two tasks."""
        self.__lock.acquire()
        self.__callback = p_callback
        self.__lock.release()

    ###########################################################################
    # ITEM HANDLING
    ###########################################################################

    def __set_current_item(self, p_item):
        self.__lock.acquire()
        self.__current_item = p_item
        self.__lock.release()

    def stop_item_handling(self):
        """Stop the handling of the current item."""
        pass
        
    def handle_item(self, p_item):
        pass

    ###########################################################################
    # THREAD MANAGEMENT
    ###########################################################################

    def __changed(self):
        """Call the callback function."""
        _function = self.get_changed_callback()
        if _function is not None:
            _function()
        
    def run(self):
        """Run the thread."""
        while 1:
            self.__event.wait()
            if self.__kill:
                break
            if self.__queue.is_empty():
                self.__event.clear()
            else:
                # pop an item from the queue
                _item = self.__queue.pop_first()
                self.__set_current_item(_item)
                self.__changed()
                # handle the item
                self.handle_item(_item)
                # set None to the current item
                self.__set_current_item(None)
                self.__changed()

    def join(self, p_delay = 1):
        """Wait until there is no item in the queue."""
        while self.is_alive() and not self.__queue.is_empty():
            threading.Thread.join(self, p_delay)
        
    def stop(self):
        """Stop the playing."""
        if self.__event.isSet():
            self.__event.clear()
            self.stop_item_handling()
        
    def restart(self):
        """Force the thread to continue its job."""
        self.__event.set()
        
    def kill(self):
        """Stop definitely the thread."""
        self.__kill = 1
        self.stop_item_handling()
        self.__event.set()

