# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import threading


class SyncList:
    """A synchronozed list."""

    __list = []

    __lock = None

    def __init__(self, p_list = None):
        self.__list = []
        self.__lock = threading.Lock()
        if p_list:
            self.extend(p_list)

    ###########################################################################
    # ACCESSORS
    ###########################################################################

    def __len__(self):
        """Return the lenght of the list."""
        self.__lock.acquire()
        _ret = len(self.__list)
        self.__lock.release()
        return _ret

    def get_length(self):
        """Return the length."""
        return len(self)

    def is_empty(self):
        """Determine if the list is empty."""
        return (self.get_length() == 0)

    def __getitem__(self, p_index):
        """Return the item at the given index."""
        self.__lock.acquire()
        try:
            _ret = self.__list[p_index]
        finally:
            self.__lock.release()
        return _ret

    def __getslice__(self, p_index_start, p_index_end):
        """Return the slice between thetwo given indexes."""
        self.__lock.acquire()
        try:
            _ret = self.__list[p_index_start:p_index_end]
        finally:
            self.__lock.release()
        return _ret

    def index(self, p_item):
        """Return the index where the given item is.

        Rase ValueError if no item is found."""
        self.__lock.acquire()
        try:
            _ret = self.__list.index(p_item)
        finally:
            self.__lock.release()
        return _ret

    def __add__(self, p_list):
        """Add the current list to the given list and return the new list."""
        _ret = SyncList(self)
        _ret.extend(p_list)
        return _ret

    def __cmp__(self, p_list):
        """Compare the given list to the current list."""
        _ret = 0
        _cpt1 = 0
        _cpt2 = 0
        while _ret == 0 and _cpt1 < len(self) and _cpt2 < len(p_list):
            _ret = cmp(self[_cpt1], p_list[_cpt2])
            _cpt1 += 1
            _cpt2 += 1
        return _ret

    def __str__(self):
        """Return the content of the list."""
        self.__lock.acquire()
        _ret = str(self.__list)
        self.__lock.release()
        return _ret

    ###########################################################################
    # MODIFICATORS
    ###########################################################################

    def __setitem__(self, p_index, p_item):
        """Set the item at the given index to the given value."""
        self.__lock.acquire()
        try:
            self.__list[p_index] = p_item
        finally:
            self.__lock.release()

    def __delitem__(self, p_index):
        """Delete the item at the given index."""
        self.__lock.acquire()
        try:
            del self.__list[p_index]
        finally:
            self.__lock.release()

    def __setslice__(self, p_index_start, p_index_end, p_list):
        """Set the slice between the two given indexes to the given list."""
        self.__lock.acquire()
        try:
            self.__list[p_index_start:p_index_end] = p_list
        finally:
            self.__lock.release()

    def __delslice__(self, p_index_start, p_index_end):
        """Delete the slice between thetwo given indexes."""
        self.__lock.acquire()
        try:
            del self.__list[p_index_start:p_index_end]
        finally:
            self.__lock.release()

    def insert(self, p_index, p_item):
        """Insert the given item at the given index."""
        self.__lock.acquire()
        try:
            self.__list.insert(p_index, p_item)
        finally:
            self.__lock.release()

    def append(self, p_item):
        """Insert the given item at the end."""
        self.__lock.acquire()
        self.__list.append(p_item)
        self.__lock.release()

    def extend(self, p_list):
        """Append all items of the given list."""
        self.__list.extend(p_list)

    def prepend(self, p_item):
        """Insert the given item at the beginning."""
        self.insert(0, p_item)

    def delete(self, p_index):
        """Remove the item at the given index.

        Raise IndexError if the index is out of bound."""
        self.__lock.acquire()
        try:
            del self.__list[p_index]
        finally:
            self.__lock.release()
            
    def pop(self, p_index):
        """Remove the item at the given index and return it.

        Raise IndexError if the list is empty."""
        self.__lock.acquire()
        try:
            _ret = self.__list.pop(p_index)
        finally:
            self.__lock.release()
        return _ret

    def pop_first(self):
        """Remove the first item and return it."""
        return self.pop(0)

    def remove(self, p_item):
        """Remove the given item.

        Raise ValueError if the item is not found."""
        self.__lock.acquire()
        try:
            self.__list.remove(p_item)
        finally:
            self.__lock.release()
            
    def clear(self):
        """Clear the list."""
        while not self.is_empty():
            self.delete(0)

    def sort(self, p_cmp_func = None):
        """Sort the list using the given comparison function."""
        self.__lock.acquire()
        try:
            if p_cmp_func:
                self.__list.sort(p_cmp_func)
            else:
                self.__list.sort()
        finally:
            self.__lock.release()




if __name__ == '__main__':
    _list = SyncList()
    print 'is_empty...', (_list.is_empty())
    print 'append 1...',
    _list.append(1)
    print _list
    print 'append 3...',
    _list.append(3)
    print _list
    print 'append 2...',
    _list.append(2)
    print _list
    print 'len...', (len(_list) == 3)
    print 'get_length...', (_list.get_length() == 3)
    print 'is_empty...', (not _list.is_empty())
    print '__getitem__...', (_list[0] == 1)
    print '__getitem__...',
    try:
        print not _list[3]
    except IndexError:
        print 1
    print 'index...', (_list.index(2) == 2)
    print 'index...',
    try:
        print not _list.index(4)
    except ValueError, _exc:
        print 1
    print '__setitem__ 1 4...',
    _list[1] = 4
    print _list
    print '__setitem__ 3 4...',
    try:
        _list[3] = 4
        print 0
    except IndexError:
        print 1
    print 'insert 1 5...',
    _list.insert(1, 5)
    print _list
    print 'extend [1, 2, 3]...',
    _list.extend([1, 2, 3])
    print _list
    print 'prepend 0...',
    _list.prepend(0)
    print _list
    print 'delete 3...',
    _list.delete(3)
    print _list
    print 'delete 15...',
    try:
        _list.delete(15)
        print 0
    except IndexError:
        print 1
    print 'pop 3...', (_list.pop(3) == 2)
    print 'pop first...', (_list.pop_first() == 0)
    print 'remove 2...',
    _list.remove(2)
    print _list
    print 'remove 7...',
    try:
        _list.remove(7)
        print 0
    except ValueError:
        print 1
    print 'sort...',
    _list.sort()
    print _list
    print '1 in...', (1 in _list)
    print '7 not in...', (7 not in _list)
    _list2 = SyncList([7, 8, 9])
    print _list2
    print _list
    print _list + _list2
    print '_list + _list2...', (_list + _list2 == [1, 1, 3, 5, 7, 8, 9])
    print 'clear...',
    _list.clear()
    print _list
    
